package org.apache.bcel.verifier.structurals;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import org.apache.bcel.generic.ATHROW;
import org.apache.bcel.generic.BranchInstruction;
import org.apache.bcel.generic.ConstantPoolGen;
import org.apache.bcel.generic.GotoInstruction;
import org.apache.bcel.generic.InstructionHandle;
import org.apache.bcel.generic.JsrInstruction;
import org.apache.bcel.generic.LocalVariableInstruction;
import org.apache.bcel.generic.MethodGen;
import org.apache.bcel.generic.RET;
import org.apache.bcel.generic.ReturnInstruction;
import org.apache.bcel.generic.Select;
import org.apache.bcel.verifier.exc.AssertionViolatedException;
import org.apache.bcel.verifier.exc.StructuralCodeConstraintException;

/* loaded from: org_apache_bcel_verifier_structurals.jar:org/apache/bcel/verifier/structurals/Subroutines.class */
public class Subroutines {
    private final Map<InstructionHandle, Subroutine> subroutines;
    public final Subroutine TOPLEVEL;

    /* loaded from: org_apache_bcel_verifier_structurals.jar:org/apache/bcel/verifier/structurals/Subroutines$ColourConstants.class */
    private enum ColourConstants {
        WHITE,
        GRAY,
        BLACK
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: org_apache_bcel_verifier_structurals.jar:org/apache/bcel/verifier/structurals/Subroutines$SubroutineImpl.class */
    public class SubroutineImpl implements Subroutine {
        private static final int UNSET = -1;
        private final SubroutineImpl[] EMPTY_ARRAY = new SubroutineImpl[0];
        private int localVariable = UNSET;
        private final Set<InstructionHandle> instructions = new HashSet();
        private final Set<InstructionHandle> theJSRs = new HashSet();
        private InstructionHandle theRET;

        public SubroutineImpl() {
        }

        private void _getRecursivelyAccessedLocalsIndicesHelper(Set<Integer> set, Subroutine[] subs) {
            for (Subroutine sub : subs) {
                int[] lvs = sub.getAccessedLocalsIndices();
                for (int lv : lvs) {
                    set.add(Integer.valueOf(lv));
                }
                if (sub.subSubs().length != 0) {
                    _getRecursivelyAccessedLocalsIndicesHelper(set, sub.subSubs());
                }
            }
        }

        public void addEnteringJsrInstruction(InstructionHandle jsrInst) {
            if (jsrInst == null || !(jsrInst.getInstruction() instanceof JsrInstruction)) {
                throw new AssertionViolatedException("Expecting JsrInstruction InstructionHandle.");
            }
            if (this.localVariable == UNSET) {
                throw new AssertionViolatedException("Set the localVariable first!");
            }
            if (this.localVariable != jsrInst.getInstruction().getTarget().getInstruction().getIndex()) {
                throw new AssertionViolatedException("Setting a wrong JsrInstruction.");
            }
            this.theJSRs.add(jsrInst);
        }

        void addInstruction(InstructionHandle ih) {
            if (this.theRET != null) {
                throw new AssertionViolatedException("All instructions must have been added before invoking setLeavingRET().");
            }
            this.instructions.add(ih);
        }

        @Override // org.apache.bcel.verifier.structurals.Subroutine
        public boolean contains(InstructionHandle inst) {
            return this.instructions.contains(inst);
        }

        @Override // org.apache.bcel.verifier.structurals.Subroutine
        public int[] getAccessedLocalsIndices() {
            Set<Integer> acc = new HashSet<>();
            if (this.theRET == null && this != Subroutines.this.getTopLevel()) {
                throw new AssertionViolatedException("This subroutine object must be built up completely before calculating accessed locals.");
            }
            for (InstructionHandle ih : this.instructions) {
                if ((ih.getInstruction() instanceof LocalVariableInstruction) || (ih.getInstruction() instanceof RET)) {
                    int idx = ih.getInstruction().getIndex();
                    acc.add(Integer.valueOf(idx));
                    try {
                        if (ih.getInstruction() instanceof LocalVariableInstruction) {
                            int s = ih.getInstruction().getType((ConstantPoolGen) null).getSize();
                            if (s == 2) {
                                acc.add(Integer.valueOf(idx + 1));
                            }
                        }
                    } catch (RuntimeException re) {
                        throw new AssertionViolatedException("BCEL did not like NULL as a ConstantPoolGen object.", re);
                    }
                }
            }
            int[] ret = new int[acc.size()];
            int j = UNSET;
            for (Integer accessedLocal : acc) {
                j++;
                ret[j] = accessedLocal.intValue();
            }
            return ret;
        }

        @Override // org.apache.bcel.verifier.structurals.Subroutine
        public InstructionHandle[] getEnteringJsrInstructions() {
            if (this == Subroutines.this.getTopLevel()) {
                throw new AssertionViolatedException("getLeavingRET() called on top level pseudo-subroutine.");
            }
            return (InstructionHandle[]) this.theJSRs.toArray(InstructionHandle.EMPTY_ARRAY);
        }

        @Override // org.apache.bcel.verifier.structurals.Subroutine
        public InstructionHandle[] getInstructions() {
            return (InstructionHandle[]) this.instructions.toArray(InstructionHandle.EMPTY_ARRAY);
        }

        @Override // org.apache.bcel.verifier.structurals.Subroutine
        public InstructionHandle getLeavingRET() {
            if (this == Subroutines.this.getTopLevel()) {
                throw new AssertionViolatedException("getLeavingRET() called on top level pseudo-subroutine.");
            }
            return this.theRET;
        }

        @Override // org.apache.bcel.verifier.structurals.Subroutine
        public int[] getRecursivelyAccessedLocalsIndices() {
            Set<Integer> s = new HashSet<>();
            int[] lvs = getAccessedLocalsIndices();
            for (int lv : lvs) {
                s.add(Integer.valueOf(lv));
            }
            _getRecursivelyAccessedLocalsIndicesHelper(s, subSubs());
            int[] ret = new int[s.size()];
            int j = UNSET;
            for (Integer index : s) {
                j++;
                ret[j] = index.intValue();
            }
            return ret;
        }

        void setLeavingRET() {
            if (this.localVariable == UNSET) {
                throw new AssertionViolatedException("setLeavingRET() called for top-level 'subroutine' or forgot to set local variable first.");
            }
            InstructionHandle ret = null;
            for (InstructionHandle actual : this.instructions) {
                if (actual.getInstruction() instanceof RET) {
                    if (ret != null) {
                        throw new StructuralCodeConstraintException("Subroutine with more then one RET detected: '" + ret + "' and '" + actual + "'.");
                    }
                    ret = actual;
                }
            }
            if (ret == null) {
                throw new StructuralCodeConstraintException("Subroutine without a RET detected.");
            }
            if (ret.getInstruction().getIndex() != this.localVariable) {
                throw new StructuralCodeConstraintException("Subroutine uses '" + ret + "' which does not match the correct local variable '" + this.localVariable + "'.");
            }
            this.theRET = ret;
        }

        void setLocalVariable(int i) {
            if (this.localVariable != UNSET) {
                throw new AssertionViolatedException("localVariable set twice.");
            }
            this.localVariable = i;
        }

        @Override // org.apache.bcel.verifier.structurals.Subroutine
        public Subroutine[] subSubs() {
            Set<Subroutine> h = new HashSet<>();
            for (InstructionHandle ih : this.instructions) {
                JsrInstruction instruction = ih.getInstruction();
                if (instruction instanceof JsrInstruction) {
                    InstructionHandle targ = instruction.getTarget();
                    h.add(Subroutines.this.getSubroutine(targ));
                }
            }
            return (Subroutine[]) h.toArray(this.EMPTY_ARRAY);
        }

        public String toString() {
            StringBuilder ret = new StringBuilder();
            ret.append("Subroutine: Local variable is '").append(this.localVariable);
            ret.append("', JSRs are '").append(this.theJSRs);
            ret.append("', RET is '").append(this.theRET);
            ret.append("', Instructions: '").append(this.instructions).append("'.");
            ret.append(" Accessed local variable slots: '");
            int[] alv = getAccessedLocalsIndices();
            for (int element : alv) {
                ret.append(element);
                ret.append(" ");
            }
            ret.append("'.");
            ret.append(" Recursively (via subsub...routines) accessed local variable slots: '");
            int[] alv2 = getRecursivelyAccessedLocalsIndices();
            for (int element2 : alv2) {
                ret.append(element2);
                ret.append(" ");
            }
            ret.append("'.");
            return ret.toString();
        }
    }

    private static InstructionHandle[] getSuccessors(InstructionHandle instruction) {
        InstructionHandle[] single = new InstructionHandle[1];
        GotoInstruction instruction2 = instruction.getInstruction();
        if ((instruction2 instanceof RET) || (instruction2 instanceof ReturnInstruction) || (instruction2 instanceof ATHROW)) {
            return InstructionHandle.EMPTY_ARRAY;
        }
        if (instruction2 instanceof JsrInstruction) {
            single[0] = instruction.getNext();
            return single;
        }
        if (instruction2 instanceof GotoInstruction) {
            single[0] = instruction2.getTarget();
            return single;
        }
        if (instruction2 instanceof BranchInstruction) {
            if (instruction2 instanceof Select) {
                InstructionHandle[] matchTargets = ((Select) instruction2).getTargets();
                InstructionHandle[] ret = new InstructionHandle[matchTargets.length + 1];
                ret[0] = ((Select) instruction2).getTarget();
                System.arraycopy(matchTargets, 0, ret, 1, matchTargets.length);
                return ret;
            }
            InstructionHandle[] pair = {instruction.getNext(), ((BranchInstruction) instruction2).getTarget()};
            return pair;
        }
        single[0] = instruction.getNext();
        return single;
    }

    public Subroutines(MethodGen mg) {
        this(mg, true);
    }

    /* JADX WARN: Code restructure failed: missing block: B:109:0x03a2, code lost:
    
        r16 = r16 + 1;
     */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
        To view partially-correct code enable 'Show inconsistent code' option in preferences
    */
    public Subroutines(org.apache.bcel.generic.MethodGen r6, boolean r7) {
        /*
            Method dump skipped, instructions count: 952
            To view this dump change 'Code comments level' option to 'DEBUG'
        */
        throw new UnsupportedOperationException("Method not decompiled: org.apache.bcel.verifier.structurals.Subroutines.<init>(org.apache.bcel.generic.MethodGen, boolean):void");
    }

    public Subroutine getSubroutine(InstructionHandle leader) {
        Subroutine ret = this.subroutines.get(leader);
        if (ret == null) {
            throw new AssertionViolatedException("Subroutine requested for an InstructionHandle that is not a leader of a subroutine.");
        }
        if (ret == this.TOPLEVEL) {
            throw new AssertionViolatedException("TOPLEVEL special subroutine requested; use getTopLevel().");
        }
        return ret;
    }

    public Subroutine getTopLevel() {
        return this.TOPLEVEL;
    }

    private void noRecursiveCalls(Subroutine sub, Set<Integer> set) {
        Subroutine[] subs = sub.subSubs();
        for (Subroutine sub2 : subs) {
            int index = sub2.getLeavingRET().getInstruction().getIndex();
            if (!set.add(Integer.valueOf(index))) {
                SubroutineImpl si = (SubroutineImpl) sub2;
                throw new StructuralCodeConstraintException("Subroutine with local variable '" + si.localVariable + "', JSRs '" + si.theJSRs + "', RET '" + si.theRET + "' is called by a subroutine which uses the same local variable index as itself; maybe even a recursive call? JustIce's clean definition of a subroutine forbids both.");
            }
            noRecursiveCalls(sub2, set);
            set.remove(Integer.valueOf(index));
        }
    }

    public Subroutine subroutineOf(InstructionHandle any) {
        for (Subroutine s : this.subroutines.values()) {
            if (s.contains(any)) {
                return s;
            }
        }
        System.err.println("DEBUG: Please verify '" + any.toString(true) + "' lies in dead code.");
        return null;
    }

    public String toString() {
        return "---\n" + this.subroutines + "\n---\n";
    }
}
